@@ -10,23 +10,7 @@
  * Copyright (c) 2021, CLS
  * Copyright (c) 2022, Planet Labs
  *
- * Permission is hereby granted, free of charge, to any person obtaining a
- * copy of this software and associated documentation files (the "Software"),
- * to deal in the Software without restriction, including without limitation
- * the rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons to whom the
- * Software is furnished to do so, subject to the following conditions:
- *
- * The above copyright notice and this permission notice shall be included
- * in all copies or substantial portions of the Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
- * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
- * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
- * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
- * DEALINGS IN THE SOFTWARE.
+ * SPDX-License-Identifier: MIT
  ****************************************************************************/
 
 #include "cpl_port.h"
@@ -605,6 +589,7 @@ int GDALGeoLoc<Accessors>::Transform(void *pTransformArg, int bDstToSrc,
                                      double *padfY, double * /* padfZ */,
                                      int *panSuccess)
 {
+    int bSuccess = TRUE;
     GDALGeoLocTransformInfo *psTransform =
         static_cast<GDALGeoLocTransformInfo *>(pTransformArg);
 
@@ -623,6 +608,7 @@ int GDALGeoLoc<Accessors>::Transform(void *pTransformArg, int bDstToSrc,
         {
             if (padfX[i] == HUGE_VAL || padfY[i] == HUGE_VAL)
             {
+                bSuccess = FALSE;
                 panSuccess[i] = FALSE;
                 continue;
             }
@@ -639,6 +625,7 @@ int GDALGeoLoc<Accessors>::Transform(void *pTransformArg, int bDstToSrc,
             if (!PixelLineToXY(psTransform, dfGeoLocPixel, dfGeoLocLine,
                                padfX[i], padfY[i]))
             {
+                bSuccess = FALSE;
                 panSuccess[i] = FALSE;
                 padfX[i] = HUGE_VAL;
                 padfY[i] = HUGE_VAL;
@@ -681,6 +668,7 @@ int GDALGeoLoc<Accessors>::Transform(void *pTransformArg, int bDstToSrc,
         {
             if (padfX[i] == HUGE_VAL || padfY[i] == HUGE_VAL)
             {
+                bSuccess = FALSE;
                 panSuccess[i] = FALSE;
                 continue;
             }
@@ -704,6 +692,7 @@ int GDALGeoLoc<Accessors>::Transform(void *pTransformArg, int bDstToSrc,
                   dfBMX + 1 < psTransform->nBackMapWidth &&
                   dfBMY + 1 < psTransform->nBackMapHeight))
             {
+                bSuccess = FALSE;
                 panSuccess[i] = FALSE;
                 padfX[i] = HUGE_VAL;
                 padfY[i] = HUGE_VAL;
@@ -717,6 +706,7 @@ int GDALGeoLoc<Accessors>::Transform(void *pTransformArg, int bDstToSrc,
             const auto fBMY_0_0 = pAccessors->backMapYAccessor.Get(iBMX, iBMY);
             if (fBMX_0_0 == INVALID_BMXY)
             {
+                bSuccess = FALSE;
                 panSuccess[i] = FALSE;
                 padfX[i] = HUGE_VAL;
                 padfY[i] = HUGE_VAL;
@@ -930,6 +920,7 @@ int GDALGeoLoc<Accessors>::Transform(void *pTransformArg, int bDstToSrc,
             }
             if (!bDone)
             {
+                bSuccess = FALSE;
                 panSuccess[i] = FALSE;
                 padfX[i] = HUGE_VAL;
                 padfY[i] = HUGE_VAL;
@@ -940,7 +931,7 @@ int GDALGeoLoc<Accessors>::Transform(void *pTransformArg, int bDstToSrc,
         }
     }
 
-    return TRUE;
+    return bSuccess;
 }
 
 /*! @endcond */
@@ -1591,9 +1582,9 @@ static void GDALGeoLocRescale(char **&papszMD, const char *pszItem,
 {
     const double dfVal =
         dfRatio * CPLAtofM(CSLFetchNameValueDef(
-                      papszMD, pszItem, CPLSPrintf("%.18g", dfDefaultVal)));
+                      papszMD, pszItem, CPLSPrintf("%.17g", dfDefaultVal)));
 
-    papszMD = CSLSetNameValue(papszMD, pszItem, CPLSPrintf("%.18g", dfVal));
+    papszMD = CSLSetNameValue(papszMD, pszItem, CPLSPrintf("%.17g", dfVal));
 }
 
 /************************************************************************/
@@ -1718,7 +1709,7 @@ CPLStringList GDALCreateGeolocationMetadata(GDALDatasetH hBaseDS,
     if (aosMD.FetchNameValue("PIXEL_STEP") == nullptr)
     {
         aosMD.SetNameValue(
-            "PIXEL_STEP", CPLSPrintf("%.18g", static_cast<double>(
+            "PIXEL_STEP", CPLSPrintf("%.17g", static_cast<double>(
                                                   GDALGetRasterXSize(hBaseDS)) /
                                                   nGeoLocXSize));
     }
@@ -1726,7 +1717,7 @@ CPLStringList GDALCreateGeolocationMetadata(GDALDatasetH hBaseDS,
     if (aosMD.FetchNameValue("LINE_STEP") == nullptr)
     {
         aosMD.SetNameValue(
-            "LINE_STEP", CPLSPrintf("%.18g", static_cast<double>(
+            "LINE_STEP", CPLSPrintf("%.17g", static_cast<double>(
                                                  GDALGetRasterYSize(hBaseDS)) /
                                                  nGeoLocYSize));
     }
@@ -1833,9 +1824,11 @@ void *GDALCreateGeoLocTransformerEx(GDALDatasetH hBaseDS,
                 papszGeolocationInfo, "X_DATASET_RELATIVE_TO_SOURCE", "NO")) &&
             (hBaseDS != nullptr || pszSourceDataset))
         {
-            CPLString osFilename = CPLProjectRelativeFilename(
-                CPLGetDirname(pszSourceDataset ? pszSourceDataset
-                                               : GDALGetDescription(hBaseDS)),
+            const CPLString osFilename = CPLProjectRelativeFilenameSafe(
+                CPLGetDirnameSafe(pszSourceDataset
+                                      ? pszSourceDataset
+                                      : GDALGetDescription(hBaseDS))
+                    .c_str(),
                 pszDSName);
             psTransform->hDS_X =
                 GDALOpenShared(osFilename.c_str(), GA_ReadOnly);
@@ -1865,9 +1858,11 @@ void *GDALCreateGeoLocTransformerEx(GDALDatasetH hBaseDS,
                 papszGeolocationInfo, "Y_DATASET_RELATIVE_TO_SOURCE", "NO")) &&
             (hBaseDS != nullptr || pszSourceDataset))
         {
-            CPLString osFilename = CPLProjectRelativeFilename(
-                CPLGetDirname(pszSourceDataset ? pszSourceDataset
-                                               : GDALGetDescription(hBaseDS)),
+            const CPLString osFilename = CPLProjectRelativeFilenameSafe(
+                CPLGetDirnameSafe(pszSourceDataset
+                                      ? pszSourceDataset
+                                      : GDALGetDescription(hBaseDS))
+                    .c_str(),
                 pszDSName);
             psTransform->hDS_Y =
                 GDALOpenShared(osFilename.c_str(), GA_ReadOnly);
@@ -2006,14 +2001,20 @@ void *GDALCreateGeoLocTransformerEx(GDALDatasetH hBaseDS,
         psTransform->bUseArray = !CPLTestBool(pszUseTempDatasets);
     else
     {
-        psTransform->bUseArray = nXSize < 16 * 1000 * 1000 / nYSize;
-        if (psTransform->bUseArray)
+        constexpr int MEGAPIXEL_LIMIT = 24;
+        psTransform->bUseArray =
+            nXSize < MEGAPIXEL_LIMIT * 1000 * 1000 / nYSize;
+        if (!psTransform->bUseArray)
         {
             CPLDebug("GEOLOC",
                      "Using temporary GTiff backing to store backmap, because "
-                     "geoloc arrays exceed 16 megapixels. You can set the "
+                     "geoloc arrays require %d megapixels, exceeding the %d "
+                     "megapixels limit. You can set the "
                      "GDAL_GEOLOC_USE_TEMP_DATASETS configuration option to "
-                     "NO to force RAM storage of backmap");
+                     "NO to force RAM storage of backmap",
+                     static_cast<int>(static_cast<int64_t>(nXSize) * nYSize /
+                                      (1000 * 1000)),
+                     MEGAPIXEL_LIMIT);
         }
     }
 
