@@ -8,23 +8,7 @@
  * Copyright (c) 2001, 2004, Frank Warmerdam <warmerdam@pobox.com>
  * Copyright (c) 2008-2013, Even Rouault <even dot rouault at spatialys.com>
  *
- * Permission is hereby granted, free of charge, to any person obtaining a
- * copy of this software and associated documentation files (the "Software"),
- * to deal in the Software without restriction, including without limitation
- * the rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons to whom the
- * Software is furnished to do so, subject to the following conditions:
- *
- * The above copyright notice and this permission notice shall be included
- * in all copies or substantial portions of the Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
- * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
- * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
- * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
- * DEALINGS IN THE SOFTWARE.
+ * SPDX-License-Identifier: MIT
  ****************************************************************************/
 
 // ncsjpcbuffer.h needs the min and max macros.
@@ -843,7 +827,8 @@ CPLErr GDALECWCompressor::Initialize(
             SetParameter(CNCSJP2FileView::JP2_COMPRESS_PROFILE_NITF_BIIF_EPJE);
 
         pszOption = CSLFetchNameValue(papszOptions, "CODESTREAM_ONLY");
-        if (pszOption == nullptr && EQUAL(CPLGetExtension(pszFilename), "j2k"))
+        if (pszOption == nullptr &&
+            EQUAL(CPLGetExtensionSafe(pszFilename).c_str(), "j2k"))
             pszOption = "YES";
         if (pszOption != nullptr)
             SetParameter(CNCSJP2FileView::JP2_COMPRESS_CODESTREAM_ONLY,
@@ -1415,16 +1400,13 @@ static GDALDataset *ECWCreateCopy(const char *pszFilename, GDALDataset *poSrcDS,
     oCompressor.pProgressData = pProgressData;
     oCompressor.m_poSrcDS = poSrcDS;
 
-    if (!pfnProgress(0.0, nullptr, pProgressData))
-        return nullptr;
-
-    char **papszBandDescriptions = (char **)CPLMalloc(nBands * sizeof(char *));
+    CPLStringList aosBandDescriptions;
     for (int i = 0; i < nBands; i++)
     {
         /* Make a copy since ECWGetColorInterpretationName() can return a string
          * generated */
         /* by CPLSPrintf(), which has just a few rotating entries. */
-        papszBandDescriptions[i] = CPLStrdup(ECWGetColorInterpretationName(
+        aosBandDescriptions.AddString(ECWGetColorInterpretationName(
             poSrcDS->GetRasterBand(i + 1)->GetColorInterpretation(), i));
     }
 
@@ -1433,29 +1415,27 @@ static GDALDataset *ECWCreateCopy(const char *pszFilename, GDALDataset *poSrcDS,
         pszAreaOrPoint != nullptr && EQUAL(pszAreaOrPoint, GDALMD_AOP_POINT);
 
     if (oCompressor.Initialize(pszFilename, papszOptions, nXSize, nYSize,
-                               nBands, papszBandDescriptions, bRGBColorSpace,
-                               eType, poSRS, adfGeoTransform,
+                               nBands, aosBandDescriptions.List(),
+                               bRGBColorSpace, eType, poSRS, adfGeoTransform,
                                poSrcDS->GetGCPCount(), poSrcDS->GetGCPs(),
                                bIsJPEG2000, bPixelIsPoint,
                                poSrcDS->GetMetadata("RPC"), poSrcDS) != CE_None)
     {
-        for (int i = 0; i < nBands; i++)
-            CPLFree(papszBandDescriptions[i]);
-        CPLFree(papszBandDescriptions);
         return nullptr;
     }
 
     /* -------------------------------------------------------------------- */
     /*      Start the compression.                                          */
     /* -------------------------------------------------------------------- */
+
+    if (!pfnProgress(0.0, nullptr, pProgressData))
+        return nullptr;
+
     CNCSError oErr = oCompressor.Write();
 
     if (oErr.GetErrorNumber() != NCS_SUCCESS)
     {
         ECWReportError(oErr);
-        for (int i = 0; i < nBands; i++)
-            CPLFree(papszBandDescriptions[i]);
-        CPLFree(papszBandDescriptions);
         return nullptr;
     }
 
@@ -1463,9 +1443,6 @@ static GDALDataset *ECWCreateCopy(const char *pszFilename, GDALDataset *poSrcDS,
     /*      Cleanup, and return read-only handle.                           */
     /* -------------------------------------------------------------------- */
     oCompressor.CloseDown();
-    for (int i = 0; i < nBands; i++)
-        CPLFree(papszBandDescriptions[i]);
-    CPLFree(papszBandDescriptions);
     pfnProgress(1.001, nullptr, pProgressData);
 
     /* -------------------------------------------------------------------- */
@@ -1534,7 +1511,7 @@ GDALDataset *ECWCreateCopyECW(const char *pszFilename, GDALDataset *poSrcDS,
         return nullptr;
     }
 
-    if (!EQUAL(CPLGetExtension(pszFilename), "ecw"))
+    if (!EQUAL(CPLGetExtensionSafe(pszFilename).c_str(), "ecw"))
     {
         CPLError(CE_Failure, CPLE_AppDefined,
                  "ECW driver does not support creating ECW files\n"
@@ -1629,7 +1606,7 @@ GDALDataset *ECWCreateCopyJPEG2000(const char *pszFilename,
         return nullptr;
     }
 
-    if (EQUAL(CPLGetExtension(pszFilename), "ecw"))
+    if (EQUAL(CPLGetExtensionSafe(pszFilename).c_str(), "ecw"))
     {
         CPLError(CE_Failure, CPLE_AppDefined,
                  "JP2ECW driver does not support creating JPEG2000 files\n"
@@ -1779,7 +1756,7 @@ class ECWWriteDataset final : public GDALDataset
     virtual CPLErr IRasterIO(GDALRWFlag eRWFlag, int nXOff, int nYOff,
                              int nXSize, int nYSize, void *pData, int nBufXSize,
                              int nBufYSize, GDALDataType eBufType,
-                             int nBandCount, int *panBandMap,
+                             int nBandCount, BANDMAP_TYPE panBandMap,
                              GSpacing nPixelSpace, GSpacing nLineSpace,
                              GSpacing nBandSpace,
                              GDALRasterIOExtraArg *psExtraArg) override;
@@ -2056,7 +2033,7 @@ CPLErr ECWWriteDataset::IRasterIO(GDALRWFlag eRWFlag, int nXOff, int nYOff,
                                   int nXSize, int nYSize, void *pData,
                                   int nBufXSize, int nBufYSize,
                                   GDALDataType eBufType, int nBandCount,
-                                  int *panBandMap, GSpacing nPixelSpace,
+                                  BANDMAP_TYPE panBandMap, GSpacing nPixelSpace,
                                   GSpacing nLineSpace, GSpacing nBandSpace,
                                   GDALRasterIOExtraArg *psExtraArg)
 {
