@@ -7,27 +7,12 @@
  ******************************************************************************
  * Copyright (c) 2024, Even Rouault <even.rouault at spatialys.com>
  *
- * Permission is hereby granted, free of charge, to any person obtaining a
- * copy of this software and associated documentation files (the "Software"),
- * to deal in the Software without restriction, including without limitation
- * the rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons to whom the
- * Software is furnished to do so, subject to the following conditions:
- *
- * The above copyright notice and this permission notice shall be included
- * in all copies or substantial portions of the Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
- * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
- * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
- * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
- * DEALINGS IN THE SOFTWARE.
+ * SPDX-License-Identifier: MIT
  ****************************************************************************/
 
 #include "cpl_minixml.h"
 #include "cpl_string.h"
+#include "gdal_utils.h"
 #include "vrtdataset.h"
 
 #include <algorithm>
@@ -222,6 +207,27 @@ CPLErr VRTProcessedDataset::XMLInit(const CPLXMLNode *psTree,
     return CE_None;
 }
 
+static bool HasScaleOffset(GDALDataset &oSrcDS)
+{
+    for (int i = 1; i <= oSrcDS.GetRasterCount(); i++)
+    {
+        int pbSuccess;
+        GDALRasterBand &oBand = *oSrcDS.GetRasterBand(i);
+        double scale = oBand.GetScale(&pbSuccess);
+        if (pbSuccess && scale != 1)
+        {
+            return true;
+        }
+        double offset = oBand.GetOffset(&pbSuccess);
+        if (pbSuccess && offset != 0)
+        {
+            return true;
+        }
+    }
+
+    return false;
+}
+
 /** Instantiate object from XML tree */
 CPLErr VRTProcessedDataset::Init(const CPLXMLNode *psTree,
                                  const char *pszVRTPathIn,
@@ -249,7 +255,7 @@ CPLErr VRTProcessedDataset::Init(const CPLXMLNode *psTree,
     {
         const bool bRelativeToVRT = CPL_TO_BOOL(
             atoi(CPLGetXMLValue(psSourceFileNameNode, "relativetoVRT", "0")));
-        const std::string osFilename = VRTDataset::BuildSourceFilename(
+        const std::string osFilename = GDALDataset::BuildFilename(
             CPLGetXMLValue(psInput, "SourceFilename", ""), pszVRTPathIn,
             bRelativeToVRT);
         m_poSrcDS.reset(GDALDataset::Open(
@@ -276,6 +282,53 @@ CPLErr VRTProcessedDataset::Init(const CPLXMLNode *psTree,
     if (!m_poSrcDS)
         return CE_Failure;
 
+    const char *pszUnscale = CPLGetXMLValue(psInput, "unscale", "AUTO");
+    bool bUnscale = false;
+    if (EQUAL(pszUnscale, "AUTO"))
+    {
+        if (HasScaleOffset(*m_poSrcDS))
+        {
+            bUnscale = true;
+        }
+    }
+    else if (EQUAL(pszUnscale, "YES") || EQUAL(pszUnscale, "ON") ||
+             EQUAL(pszUnscale, "TRUE") || EQUAL(pszUnscale, "1"))
+    {
+        bUnscale = true;
+    }
+    else if (!(EQUAL(pszUnscale, "NO") || EQUAL(pszUnscale, "OFF") ||
+               EQUAL(pszUnscale, "FALSE") || EQUAL(pszUnscale, "0")))
+    {
+        CPLError(CE_Failure, CPLE_AppDefined, "Invalid value of 'unscale'");
+        return CE_Failure;
+    }
+
+    if (bUnscale)
+    {
+        CPLStringList oArgs;
+        oArgs.AddString("-unscale");
+        oArgs.AddString("-ot");
+        oArgs.AddString("Float64");
+        oArgs.AddString("-of");
+        oArgs.AddString("VRT");
+        oArgs.AddString("-a_nodata");
+        oArgs.AddString("nan");
+        auto *poArgs = GDALTranslateOptionsNew(oArgs.List(), nullptr);
+        int pbUsageError;
+        CPLAssert(poArgs);
+        m_poVRTSrcDS.reset(m_poSrcDS.release());
+        // https://trac.cppcheck.net/ticket/11325
+        // cppcheck-suppress accessMoved
+        m_poSrcDS.reset(GDALDataset::FromHandle(
+            GDALTranslate("", m_poVRTSrcDS.get(), poArgs, &pbUsageError)));
+        GDALTranslateOptionsFree(poArgs);
+
+        if (pbUsageError || !m_poSrcDS)
+        {
+            return CE_Failure;
+        }
+    }
+
     if (nRasterXSize == 0 && nRasterYSize == 0)
     {
         nRasterXSize = m_poSrcDS->GetRasterXSize();
@@ -317,12 +370,29 @@ CPLErr VRTProcessedDataset::Init(const CPLXMLNode *psTree,
 
     const auto poSrcFirstBand = m_poSrcDS->GetRasterBand(1);
     poSrcFirstBand->GetBlockSize(&m_nBlockXSize, &m_nBlockYSize);
+    bool bUserBlockSize = false;
     if (const char *pszBlockXSize =
             CPLGetXMLValue(psTree, "BlockXSize", nullptr))
+    {
+        bUserBlockSize = true;
         m_nBlockXSize = atoi(pszBlockXSize);
+        if (m_nBlockXSize <= 1)
+        {
+            CPLError(CE_Failure, CPLE_AppDefined, "Invalid BlockXSize");
+            return CE_Failure;
+        }
+    }
     if (const char *pszBlockYSize =
             CPLGetXMLValue(psTree, "BlockYSize", nullptr))
+    {
+        bUserBlockSize = true;
         m_nBlockYSize = atoi(pszBlockYSize);
+        if (m_nBlockYSize <= 1)
+        {
+            CPLError(CE_Failure, CPLE_AppDefined, "Invalid BlockYSize");
+            return CE_Failure;
+        }
+    }
 
     // Initialize all the general VRT stuff.
     if (VRTDataset::XMLInit(psTree, pszVRTPathIn) != CE_None)
@@ -351,18 +421,80 @@ CPLErr VRTProcessedDataset::Init(const CPLXMLNode *psTree,
             static_cast<double>(poParentDS->GetRasterYSize()) / nRasterYSize;
     }
 
-    // Create bands automatically from source dataset if not explicitly defined
-    // in VRT.
-    if (!CPLGetXMLNode(psTree, "VRTRasterBand"))
+    const CPLXMLNode *psOutputBands = CPLGetXMLNode(psTree, "OutputBands");
+    if (psOutputBands)
     {
-        for (int i = 0; i < m_poSrcDS->GetRasterCount(); ++i)
+        if (const char *pszCount =
+                CPLGetXMLValue(psOutputBands, "count", nullptr))
         {
-            const auto poSrcBand = m_poSrcDS->GetRasterBand(i + 1);
-            auto poBand = new VRTProcessedRasterBand(
-                this, i + 1, poSrcBand->GetRasterDataType());
-            poBand->CopyCommonInfoFrom(poSrcBand);
-            SetBand(i + 1, poBand);
+            if (EQUAL(pszCount, "FROM_LAST_STEP"))
+            {
+                m_outputBandCountProvenance = ValueProvenance::FROM_LAST_STEP;
+            }
+            else if (!EQUAL(pszCount, "FROM_SOURCE"))
+            {
+                if (CPLGetValueType(pszCount) == CPL_VALUE_INTEGER)
+                {
+                    m_outputBandCountProvenance =
+                        ValueProvenance::USER_PROVIDED;
+                    m_outputBandCountValue = atoi(pszCount);
+                    if (!GDALCheckBandCount(m_outputBandCountValue,
+                                            /* bIsZeroAllowed = */ false))
+                    {
+                        // Error emitted by above function
+                        return CE_Failure;
+                    }
+                }
+                else
+                {
+                    CPLError(CE_Failure, CPLE_AppDefined,
+                             "Invalid value for OutputBands.count");
+                    return CE_Failure;
+                }
+            }
         }
+
+        if (const char *pszType =
+                CPLGetXMLValue(psOutputBands, "dataType", nullptr))
+        {
+            if (EQUAL(pszType, "FROM_LAST_STEP"))
+            {
+                m_outputBandDataTypeProvenance =
+                    ValueProvenance::FROM_LAST_STEP;
+            }
+            else if (!EQUAL(pszType, "FROM_SOURCE"))
+            {
+                m_outputBandDataTypeProvenance = ValueProvenance::USER_PROVIDED;
+                m_outputBandDataTypeValue = GDALGetDataTypeByName(pszType);
+                if (m_outputBandDataTypeValue == GDT_Unknown)
+                {
+                    CPLError(CE_Failure, CPLE_AppDefined,
+                             "Invalid value for OutputBands.dataType");
+                    return CE_Failure;
+                }
+            }
+        }
+    }
+    else if (CPLGetXMLNode(psTree, "VRTRasterBand"))
+    {
+        m_outputBandCountProvenance = ValueProvenance::FROM_VRTRASTERBAND;
+        m_outputBandDataTypeProvenance = ValueProvenance::FROM_VRTRASTERBAND;
+    }
+
+    int nOutputBandCount = 0;
+    switch (m_outputBandCountProvenance)
+    {
+        case ValueProvenance::USER_PROVIDED:
+            nOutputBandCount = m_outputBandCountValue;
+            break;
+        case ValueProvenance::FROM_SOURCE:
+            nOutputBandCount = m_poSrcDS->GetRasterCount();
+            break;
+        case ValueProvenance::FROM_VRTRASTERBAND:
+            nOutputBandCount = nBands;
+            break;
+        case ValueProvenance::FROM_LAST_STEP:
+            break;
     }
 
     const CPLXMLNode *psProcessingSteps =
@@ -426,14 +558,39 @@ CPLErr VRTProcessedDataset::Init(const CPLXMLNode *psTree,
             {
                 // Initialize adfOutNoData with nodata value of *output* bands
                 // for final step
-                for (int i = 1; i <= nBands; ++i)
+                if (m_outputBandCountProvenance ==
+                    ValueProvenance::FROM_VRTRASTERBAND)
                 {
-                    int bHasVal = FALSE;
-                    const double dfVal =
-                        GetRasterBand(i)->GetNoDataValue(&bHasVal);
-                    adfOutNoData.emplace_back(
-                        bHasVal ? dfVal
-                                : std::numeric_limits<double>::quiet_NaN());
+                    for (int i = 1; i <= nBands; ++i)
+                    {
+                        int bHasVal = FALSE;
+                        const double dfVal =
+                            GetRasterBand(i)->GetNoDataValue(&bHasVal);
+                        adfOutNoData.emplace_back(
+                            bHasVal ? dfVal
+                                    : std::numeric_limits<double>::quiet_NaN());
+                    }
+                }
+                else if (m_outputBandCountProvenance ==
+                         ValueProvenance::FROM_SOURCE)
+                {
+                    for (int i = 1; i <= m_poSrcDS->GetRasterCount(); ++i)
+                    {
+                        int bHasVal = FALSE;
+                        const double dfVal =
+                            m_poSrcDS->GetRasterBand(i)->GetNoDataValue(
+                                &bHasVal);
+                        adfOutNoData.emplace_back(
+                            bHasVal ? dfVal
+                                    : std::numeric_limits<double>::quiet_NaN());
+                    }
+                }
+                else if (m_outputBandCountProvenance ==
+                         ValueProvenance::USER_PROVIDED)
+                {
+                    adfOutNoData.resize(
+                        m_outputBandCountValue,
+                        std::numeric_limits<double>::quiet_NaN());
                 }
             }
             if (!ParseStep(psStep, bIsFinalStep, eCurrentDT, nCurrentBandCount,
@@ -450,14 +607,163 @@ CPLErr VRTProcessedDataset::Init(const CPLXMLNode *psTree,
         return CE_Failure;
     }
 
-    if (nCurrentBandCount != nBands)
+    int nLargestInDTSizeTimesBand = 1;
+    int nLargestOutDTSizeTimesBand = 1;
+    for (const auto &oStep : m_aoSteps)
     {
-        CPLError(CE_Failure, CPLE_AppDefined,
-                 "Number of output bands of last step is not consistent with "
-                 "number of VRTProcessedRasterBand's");
+        const int nInDTSizeTimesBand =
+            GDALGetDataTypeSizeBytes(oStep.eInDT) * oStep.nInBands;
+        nLargestInDTSizeTimesBand =
+            std::max(nLargestInDTSizeTimesBand, nInDTSizeTimesBand);
+        const int nOutDTSizeTimesBand =
+            GDALGetDataTypeSizeBytes(oStep.eOutDT) * oStep.nOutBands;
+        nLargestOutDTSizeTimesBand =
+            std::max(nLargestOutDTSizeTimesBand, nOutDTSizeTimesBand);
+    }
+    m_nWorkingBytesPerPixel =
+        nLargestInDTSizeTimesBand + nLargestOutDTSizeTimesBand;
+
+    // Use only up to 40% of RAM to acquire source bands and generate the output
+    // buffer.
+    m_nAllowedRAMUsage = CPLGetUsablePhysicalRAM() / 10 * 4;
+    // Only for tests now
+    const char *pszMAX_RAM = "VRT_PROCESSED_DATASET_ALLOWED_RAM_USAGE";
+    if (const char *pszVal = CPLGetConfigOption(pszMAX_RAM, nullptr))
+    {
+        CPL_IGNORE_RET_VAL(
+            CPLParseMemorySize(pszVal, &m_nAllowedRAMUsage, nullptr));
+    }
+
+    if (m_nAllowedRAMUsage > 0)
+    {
+        bool bBlockSizeModified = false;
+        while ((m_nBlockXSize >= 2 || m_nBlockYSize >= 2) &&
+               static_cast<GIntBig>(m_nBlockXSize) * m_nBlockYSize >
+                   m_nAllowedRAMUsage / m_nWorkingBytesPerPixel)
+        {
+            if ((m_nBlockXSize == nRasterXSize ||
+                 m_nBlockYSize >= m_nBlockXSize) &&
+                m_nBlockYSize >= 2)
+            {
+                m_nBlockYSize /= 2;
+            }
+            else
+            {
+                m_nBlockXSize /= 2;
+            }
+            bBlockSizeModified = true;
+        }
+        if (bBlockSizeModified)
+        {
+            if (bUserBlockSize)
+            {
+                CPLError(
+                    CE_Warning, CPLE_AppDefined,
+                    "Reducing block size to %d x %d to avoid consuming too "
+                    "much RAM",
+                    m_nBlockXSize, m_nBlockYSize);
+            }
+            else
+            {
+                CPLDebug(
+                    "VRT",
+                    "Reducing block size to %d x %d to avoid consuming too "
+                    "much RAM",
+                    m_nBlockXSize, m_nBlockYSize);
+            }
+        }
+    }
+
+    if (m_outputBandCountProvenance == ValueProvenance::FROM_LAST_STEP)
+    {
+        nOutputBandCount = nCurrentBandCount;
+    }
+    else if (nOutputBandCount != nCurrentBandCount)
+    {
+        // Should not happen frequently as pixel init functions are expected
+        // to validate that they can accept the number of output bands provided
+        // to them
+        CPLError(
+            CE_Failure, CPLE_AppDefined,
+            "Number of output bands of last step (%d) is not consistent with "
+            "number of VRTProcessedRasterBand's (%d)",
+            nCurrentBandCount, nBands);
         return CE_Failure;
     }
 
+    if (m_outputBandDataTypeProvenance == ValueProvenance::FROM_LAST_STEP)
+    {
+        m_outputBandDataTypeValue = eCurrentDT;
+    }
+
+    const auto ClearBands = [this]()
+    {
+        for (int i = 0; i < nBands; ++i)
+            delete papoBands[i];
+        CPLFree(papoBands);
+        papoBands = nullptr;
+        nBands = 0;
+    };
+
+    if (nBands != 0 &&
+        (nBands != nOutputBandCount ||
+         (m_outputBandDataTypeProvenance == ValueProvenance::FROM_LAST_STEP &&
+          m_outputBandDataTypeValue != papoBands[0]->GetRasterDataType())))
+    {
+        ClearBands();
+    }
+
+    const auto GetOutputBandType = [this, eCurrentDT](GDALDataType eSourceDT)
+    {
+        if (m_outputBandDataTypeProvenance == ValueProvenance::FROM_LAST_STEP)
+            return eCurrentDT;
+        else if (m_outputBandDataTypeProvenance ==
+                 ValueProvenance::USER_PROVIDED)
+            return m_outputBandDataTypeValue;
+        else
+            return eSourceDT;
+    };
+
+    if (m_outputBandCountProvenance == ValueProvenance::FROM_SOURCE)
+    {
+        for (int i = 0; i < m_poSrcDS->GetRasterCount(); ++i)
+        {
+            const auto poSrcBand = m_poSrcDS->GetRasterBand(i + 1);
+            const GDALDataType eOutputBandType =
+                GetOutputBandType(poSrcBand->GetRasterDataType());
+            auto poBand =
+                new VRTProcessedRasterBand(this, i + 1, eOutputBandType);
+            poBand->CopyCommonInfoFrom(poSrcBand);
+            SetBand(i + 1, poBand);
+        }
+    }
+    else if (m_outputBandCountProvenance != ValueProvenance::FROM_VRTRASTERBAND)
+    {
+        const GDALDataType eOutputBandType = GetOutputBandType(eInDT);
+
+        bool bClearAndSetBands = true;
+        if (nBands == nOutputBandCount)
+        {
+            bClearAndSetBands = false;
+            for (int i = 0; i < nBands; ++i)
+            {
+                bClearAndSetBands =
+                    bClearAndSetBands ||
+                    !dynamic_cast<VRTProcessedRasterBand *>(papoBands[i]) ||
+                    papoBands[i]->GetRasterDataType() != eOutputBandType;
+            }
+        }
+        if (bClearAndSetBands)
+        {
+            ClearBands();
+            for (int i = 0; i < nOutputBandCount; ++i)
+            {
+                SetBand(i + 1, std::make_unique<VRTProcessedRasterBand>(
+                                   this, i + 1, eOutputBandType));
+            }
+        }
+    }
+
     if (nBands > 1)
         SetMetadataItem("INTERLEAVE", "PIXEL", "IMAGE_STRUCTURE");
 
@@ -573,7 +879,7 @@ bool VRTProcessedDataset::ParseStep(const CPLXMLNode *psStep, bool bIsFinalStep,
         if (bHasVal)
         {
             oStep.aosArguments.AddNameValue("nodata",
-                                            CPLSPrintf("%.18g", dfVal));
+                                            CPLSPrintf("%.17g", dfVal));
         }
     }
 
@@ -586,7 +892,7 @@ bool VRTProcessedDataset::ParseStep(const CPLXMLNode *psStep, bool bIsFinalStep,
             const double dfVal = GetRasterBand(i)->GetOffset(&bHasVal);
             oStep.aosArguments.AddNameValue(
                 CPLSPrintf("offset_%d", i),
-                CPLSPrintf("%.18g", bHasVal ? dfVal : 0.0));
+                CPLSPrintf("%.17g", bHasVal ? dfVal : 0.0));
         }
     }
 
@@ -599,7 +905,7 @@ bool VRTProcessedDataset::ParseStep(const CPLXMLNode *psStep, bool bIsFinalStep,
             const double dfVal = GetRasterBand(i)->GetScale(&bHasVal);
             oStep.aosArguments.AddNameValue(
                 CPLSPrintf("scale_%d", i),
-                CPLSPrintf("%.18g", bHasVal ? dfVal : 1.0));
+                CPLSPrintf("%.17g", bHasVal ? dfVal : 1.0));
         }
     }
 
@@ -738,13 +1044,13 @@ bool VRTProcessedDataset::ParseStep(const CPLXMLNode *psStep, bool bIsFinalStep,
     if (oFunc.pfnInit)
     {
         double *padfOutNoData = nullptr;
-        if (bIsFinalStep)
+        if (bIsFinalStep && !adfOutNoData.empty())
         {
-            oStep.nOutBands = nBands;
-            padfOutNoData =
-                static_cast<double *>(CPLMalloc(nBands * sizeof(double)));
-            CPLAssert(adfOutNoData.size() == static_cast<size_t>(nBands));
-            memcpy(padfOutNoData, adfOutNoData.data(), nBands * sizeof(double));
+            oStep.nOutBands = static_cast<int>(adfOutNoData.size());
+            padfOutNoData = static_cast<double *>(
+                CPLMalloc(adfOutNoData.size() * sizeof(double)));
+            memcpy(padfOutNoData, adfOutNoData.data(),
+                   adfOutNoData.size() * sizeof(double));
         }
         else
         {
@@ -770,8 +1076,8 @@ bool VRTProcessedDataset::ParseStep(const CPLXMLNode *psStep, bool bIsFinalStep,
 
         if (padfOutNoData)
         {
-            adfOutNoData =
-                std::vector<double>(padfOutNoData, padfOutNoData + nBands);
+            adfOutNoData = std::vector<double>(padfOutNoData,
+                                               padfOutNoData + oStep.nOutBands);
         }
         else
         {
@@ -905,37 +1211,117 @@ void VRTProcessedDataset::GetBlockSize(int *pnBlockXSize,
  * The output is stored in m_abyInput in a pixel-interleaved way.
  */
 bool VRTProcessedDataset::ProcessRegion(int nXOff, int nYOff, int nBufXSize,
-                                        int nBufYSize)
+                                        int nBufYSize,
+                                        GDALProgressFunc pfnProgress,
+                                        void *pProgressData)
 {
 
     CPLAssert(!m_aoSteps.empty());
 
+    const size_t nPixelCount = static_cast<size_t>(nBufXSize) * nBufYSize;
+
     const int nFirstBandCount = m_aoSteps.front().nInBands;
     CPLAssert(nFirstBandCount == m_poSrcDS->GetRasterCount());
     const GDALDataType eFirstDT = m_aoSteps.front().eInDT;
     const int nFirstDTSize = GDALGetDataTypeSizeBytes(eFirstDT);
     auto &abyInput = m_abyInput;
     auto &abyOutput = m_abyOutput;
-    try
-    {
-        abyInput.resize(static_cast<size_t>(nBufXSize) * nBufYSize *
-                        nFirstBandCount * nFirstDTSize);
-    }
-    catch (const std::bad_alloc &)
-    {
-        CPLError(CE_Failure, CPLE_OutOfMemory,
-                 "Out of memory allocating working buffer");
-        return false;
-    }
 
-    if (m_poSrcDS->RasterIO(
-            GF_Read, nXOff, nYOff, nBufXSize, nBufYSize, abyInput.data(),
-            nBufXSize, nBufYSize, eFirstDT, nFirstBandCount, nullptr,
-            static_cast<GSpacing>(nFirstDTSize) * nFirstBandCount,
-            static_cast<GSpacing>(nFirstDTSize) * nFirstBandCount * nBufXSize,
-            nFirstDTSize, nullptr) != CE_None)
+    const char *pszInterleave =
+        m_poSrcDS->GetMetadataItem("INTERLEAVE", "IMAGE_STRUCTURE");
+    if (nFirstBandCount > 1 && (!pszInterleave || EQUAL(pszInterleave, "BAND")))
     {
-        return false;
+        // If there are several bands and the source dataset organization
+        // is apparently band interleaved, then first acquire data in
+        // a BSQ organization in the abyInput array use in the native
+        // data type.
+        // And then transpose it and convert it to the expected data type
+        // of the first step.
+        const auto eSrcDT = m_poSrcDS->GetRasterBand(1)->GetRasterDataType();
+        try
+        {
+            abyInput.resize(nPixelCount * nFirstBandCount *
+                            GDALGetDataTypeSizeBytes(eSrcDT));
+        }
+        catch (const std::bad_alloc &)
+        {
+            CPLError(CE_Failure, CPLE_OutOfMemory,
+                     "Out of memory allocating working buffer");
+            return false;
+        }
+
+        try
+        {
+            abyOutput.resize(nPixelCount * nFirstBandCount * nFirstDTSize);
+        }
+        catch (const std::bad_alloc &)
+        {
+            CPLError(CE_Failure, CPLE_OutOfMemory,
+                     "Out of memory allocating working buffer");
+            return false;
+        }
+
+        GDALRasterIOExtraArg sArg;
+        INIT_RASTERIO_EXTRA_ARG(sArg);
+        sArg.pfnProgress = GDALScaledProgress;
+        sArg.pProgressData =
+            GDALCreateScaledProgress(0, 0.5, pfnProgress, pProgressData);
+        if (sArg.pProgressData == nullptr)
+            sArg.pfnProgress = nullptr;
+
+        CPLDebugOnly("VRT", "ProcessRegion(): start RasterIO()");
+        const bool bOK =
+            m_poSrcDS->RasterIO(GF_Read, nXOff, nYOff, nBufXSize, nBufYSize,
+                                abyInput.data(), nBufXSize, nBufYSize, eSrcDT,
+                                nFirstBandCount, nullptr, 0, 0, 0,
+                                &sArg) == CE_None;
+        CPLDebugOnly("VRT", "ProcessRegion(): end RasterIO()");
+        GDALDestroyScaledProgress(sArg.pProgressData);
+        if (!bOK)
+            return false;
+
+        CPLDebugOnly("VRT", "ProcessRegion(): start GDALTranspose2D()");
+        GDALTranspose2D(abyInput.data(), eSrcDT, abyOutput.data(), eFirstDT,
+                        static_cast<size_t>(nBufXSize) * nBufYSize,
+                        nFirstBandCount);
+        CPLDebugOnly("VRT", "ProcessRegion(): end GDALTranspose2D()");
+
+        // Swap arrays
+        std::swap(abyInput, abyOutput);
+    }
+    else
+    {
+        try
+        {
+            abyInput.resize(nPixelCount * nFirstBandCount * nFirstDTSize);
+        }
+        catch (const std::bad_alloc &)
+        {
+            CPLError(CE_Failure, CPLE_OutOfMemory,
+                     "Out of memory allocating working buffer");
+            return false;
+        }
+
+        GDALRasterIOExtraArg sArg;
+        INIT_RASTERIO_EXTRA_ARG(sArg);
+        sArg.pfnProgress = GDALScaledProgress;
+        sArg.pProgressData =
+            GDALCreateScaledProgress(0, 0.5, pfnProgress, pProgressData);
+        if (sArg.pProgressData == nullptr)
+            sArg.pfnProgress = nullptr;
+
+        const bool bOK =
+            m_poSrcDS->RasterIO(
+                GF_Read, nXOff, nYOff, nBufXSize, nBufYSize, abyInput.data(),
+                nBufXSize, nBufYSize, eFirstDT, nFirstBandCount, nullptr,
+                static_cast<GSpacing>(nFirstDTSize) * nFirstBandCount,
+                static_cast<GSpacing>(nFirstDTSize) * nFirstBandCount *
+                    nBufXSize,
+                nFirstDTSize, &sArg) == CE_None;
+
+        GDALDestroyScaledProgress(sArg.pProgressData);
+        if (!bOK)
+            return false;
     }
 
     const double dfSrcXOff = nXOff;
@@ -956,6 +1342,8 @@ bool VRTProcessedDataset::ProcessRegion(int nXOff, int nYOff, int nBufXSize,
 
     GDALDataType eLastDT = eFirstDT;
     const auto &oMapFunctions = GetGlobalMapProcessedDatasetFunc();
+
+    int iStep = 0;
     for (const auto &oStep : m_aoSteps)
     {
         const auto oIterFunc = oMapFunctions.find(oStep.osAlgorithm);
@@ -966,8 +1354,7 @@ bool VRTProcessedDataset::ProcessRegion(int nXOff, int nYOff, int nBufXSize,
         {
             try
             {
-                abyOutput.resize(static_cast<size_t>(nBufXSize) * nBufYSize *
-                                 oStep.nInBands *
+                abyOutput.resize(nPixelCount * oStep.nInBands *
                                  GDALGetDataTypeSizeBytes(oStep.eInDT));
             }
             catch (const std::bad_alloc &)
@@ -980,16 +1367,14 @@ bool VRTProcessedDataset::ProcessRegion(int nXOff, int nYOff, int nBufXSize,
             GDALCopyWords64(abyInput.data(), eLastDT,
                             GDALGetDataTypeSizeBytes(eLastDT), abyOutput.data(),
                             oStep.eInDT, GDALGetDataTypeSizeBytes(oStep.eInDT),
-                            static_cast<size_t>(nBufXSize) * nBufYSize *
-                                oStep.nInBands);
+                            nPixelCount * oStep.nInBands);
 
             std::swap(abyInput, abyOutput);
         }
 
         try
         {
-            abyOutput.resize(static_cast<size_t>(nBufXSize) * nBufYSize *
-                             oStep.nOutBands *
+            abyOutput.resize(nPixelCount * oStep.nOutBands *
                              GDALGetDataTypeSizeBytes(oStep.eOutDT));
         }
         catch (const std::bad_alloc &)
@@ -1015,6 +1400,12 @@ bool VRTProcessedDataset::ProcessRegion(int nXOff, int nYOff, int nBufXSize,
 
         std::swap(abyInput, abyOutput);
         eLastDT = oStep.eOutDT;
+
+        ++iStep;
+        if (pfnProgress &&
+            !pfnProgress(0.5 + 0.5 * iStep / static_cast<int>(m_aoSteps.size()),
+                         "", pProgressData))
+            return false;
     }
 
     return true;
@@ -1078,7 +1469,8 @@ CPLErr VRTProcessedRasterBand::IReadBlock(int nBlockXOff, int nBlockYOff,
 
     const int nXPixelOff = nBlockXOff * nBlockXSize;
     const int nYPixelOff = nBlockYOff * nBlockYSize;
-    if (!poVRTDS->ProcessRegion(nXPixelOff, nYPixelOff, nBufXSize, nBufYSize))
+    if (!poVRTDS->ProcessRegion(nXPixelOff, nYPixelOff, nBufXSize, nBufYSize,
+                                nullptr, nullptr))
     {
         return CE_Failure;
     }
@@ -1115,14 +1507,14 @@ CPLErr VRTProcessedRasterBand::IReadBlock(int nBlockXOff, int nBlockYOff,
         }
         for (int iY = 0; iY < nBufYSize; ++iY)
         {
-            GDALCopyWords(poVRTDS->m_abyInput.data() +
-                              (iDstBand + static_cast<size_t>(iY) * nBufXSize *
-                                              nOutBands) *
-                                  nLastDTSize,
-                          eLastDT, nLastDTSize * nOutBands,
-                          pDst +
-                              static_cast<size_t>(iY) * nBlockXSize * nDTSize,
-                          eDataType, nDTSize, nBufXSize);
+            GDALCopyWords64(poVRTDS->m_abyInput.data() +
+                                (iDstBand + static_cast<size_t>(iY) *
+                                                nBufXSize * nOutBands) *
+                                    nLastDTSize,
+                            eLastDT, nLastDTSize * nOutBands,
+                            pDst +
+                                static_cast<size_t>(iY) * nBlockXSize * nDTSize,
+                            eDataType, nDTSize, nBufXSize);
         }
         if (poBlock)
             poBlock->DropLock();
@@ -1131,6 +1523,189 @@ CPLErr VRTProcessedRasterBand::IReadBlock(int nBlockXOff, int nBlockYOff,
     return CE_None;
 }
 
+/************************************************************************/
+/*                VRTProcessedDataset::IRasterIO()                      */
+/************************************************************************/
+
+CPLErr VRTProcessedDataset::IRasterIO(
+    GDALRWFlag eRWFlag, int nXOff, int nYOff, int nXSize, int nYSize,
+    void *pData, int nBufXSize, int nBufYSize, GDALDataType eBufType,
+    int nBandCount, BANDMAP_TYPE panBandMap, GSpacing nPixelSpace,
+    GSpacing nLineSpace, GSpacing nBandSpace, GDALRasterIOExtraArg *psExtraArg)
+{
+    // Try to pass the request to the most appropriate overview dataset.
+    if (nBufXSize < nXSize && nBufYSize < nYSize)
+    {
+        int bTried = FALSE;
+        const CPLErr eErr = TryOverviewRasterIO(
+            eRWFlag, nXOff, nYOff, nXSize, nYSize, pData, nBufXSize, nBufYSize,
+            eBufType, nBandCount, panBandMap, nPixelSpace, nLineSpace,
+            nBandSpace, psExtraArg, &bTried);
+        if (bTried)
+            return eErr;
+    }
+
+    // Optimize reading of all bands at nominal resolution for BIP-like or
+    // BSQ-like buffer spacing.
+    if (eRWFlag == GF_Read && nXSize == nBufXSize && nYSize == nBufYSize &&
+        nBandCount == nBands)
+    {
+        const auto IsSequentialBandMap = [panBandMap, nBandCount]()
+        {
+            for (int i = 0; i < nBandCount; ++i)
+            {
+                if (panBandMap[i] != i + 1)
+                {
+                    return false;
+                }
+            }
+            return true;
+        };
+
+        const int nBufTypeSize = GDALGetDataTypeSizeBytes(eBufType);
+        const bool bIsBIPLike =
+            nBandSpace == nBufTypeSize && nPixelSpace == nBandSpace * nBands &&
+            nLineSpace >= nPixelSpace * nBufXSize && IsSequentialBandMap();
+        const bool bIsBSQLike = nPixelSpace == nBufTypeSize &&
+                                nLineSpace >= nPixelSpace * nBufXSize &&
+                                nBandSpace >= nLineSpace * nBufYSize &&
+                                IsSequentialBandMap();
+        if (bIsBIPLike || bIsBSQLike)
+        {
+            GByte *pabyData = static_cast<GByte *>(pData);
+            // If acquiring the region of interest in a single time is going
+            // to consume too much RAM, split in halves.
+            if (m_nAllowedRAMUsage > 0 &&
+                static_cast<GIntBig>(nBufXSize) * nBufYSize >
+                    m_nAllowedRAMUsage / m_nWorkingBytesPerPixel)
+            {
+                if ((nBufXSize == nRasterXSize || nBufYSize >= nBufXSize) &&
+                    nBufYSize >= 2)
+                {
+                    GDALRasterIOExtraArg sArg;
+                    INIT_RASTERIO_EXTRA_ARG(sArg);
+                    const int nHalfHeight = nBufYSize / 2;
+
+                    sArg.pfnProgress = GDALScaledProgress;
+                    sArg.pProgressData = GDALCreateScaledProgress(
+                        0, 0.5, psExtraArg->pfnProgress,
+                        psExtraArg->pProgressData);
+                    if (sArg.pProgressData == nullptr)
+                        sArg.pfnProgress = nullptr;
+                    bool bOK =
+                        IRasterIO(eRWFlag, nXOff, nYOff, nBufXSize, nHalfHeight,
+                                  pabyData, nBufXSize, nHalfHeight, eBufType,
+                                  nBandCount, panBandMap, nPixelSpace,
+                                  nLineSpace, nBandSpace, &sArg) == CE_None;
+                    GDALDestroyScaledProgress(sArg.pProgressData);
+
+                    if (bOK)
+                    {
+                        sArg.pfnProgress = GDALScaledProgress;
+                        sArg.pProgressData = GDALCreateScaledProgress(
+                            0.5, 1, psExtraArg->pfnProgress,
+                            psExtraArg->pProgressData);
+                        if (sArg.pProgressData == nullptr)
+                            sArg.pfnProgress = nullptr;
+                        bOK = IRasterIO(eRWFlag, nXOff, nYOff + nHalfHeight,
+                                        nBufXSize, nBufYSize - nHalfHeight,
+                                        pabyData + nHalfHeight * nLineSpace,
+                                        nBufXSize, nBufYSize - nHalfHeight,
+                                        eBufType, nBandCount, panBandMap,
+                                        nPixelSpace, nLineSpace, nBandSpace,
+                                        &sArg) == CE_None;
+                        GDALDestroyScaledProgress(sArg.pProgressData);
+                    }
+                    return bOK ? CE_None : CE_Failure;
+                }
+                else if (nBufXSize >= 2)
+                {
+                    GDALRasterIOExtraArg sArg;
+                    INIT_RASTERIO_EXTRA_ARG(sArg);
+                    const int nHalfWidth = nBufXSize / 2;
+
+                    sArg.pfnProgress = GDALScaledProgress;
+                    sArg.pProgressData = GDALCreateScaledProgress(
+                        0, 0.5, psExtraArg->pfnProgress,
+                        psExtraArg->pProgressData);
+                    if (sArg.pProgressData == nullptr)
+                        sArg.pfnProgress = nullptr;
+                    bool bOK =
+                        IRasterIO(eRWFlag, nXOff, nYOff, nHalfWidth, nBufYSize,
+                                  pabyData, nHalfWidth, nBufYSize, eBufType,
+                                  nBandCount, panBandMap, nPixelSpace,
+                                  nLineSpace, nBandSpace, &sArg) == CE_None;
+                    GDALDestroyScaledProgress(sArg.pProgressData);
+
+                    if (bOK)
+                    {
+                        sArg.pfnProgress = GDALScaledProgress;
+                        sArg.pProgressData = GDALCreateScaledProgress(
+                            0.5, 1, psExtraArg->pfnProgress,
+                            psExtraArg->pProgressData);
+                        if (sArg.pProgressData == nullptr)
+                            sArg.pfnProgress = nullptr;
+                        bOK = IRasterIO(eRWFlag, nXOff + nHalfWidth, nYOff,
+                                        nBufXSize - nHalfWidth, nBufYSize,
+                                        pabyData + nHalfWidth * nPixelSpace,
+                                        nBufXSize - nHalfWidth, nBufYSize,
+                                        eBufType, nBandCount, panBandMap,
+                                        nPixelSpace, nLineSpace, nBandSpace,
+                                        &sArg) == CE_None;
+                        GDALDestroyScaledProgress(sArg.pProgressData);
+                    }
+                    return bOK ? CE_None : CE_Failure;
+                }
+            }
+
+            if (!ProcessRegion(nXOff, nYOff, nBufXSize, nBufYSize,
+                               psExtraArg->pfnProgress,
+                               psExtraArg->pProgressData))
+            {
+                return CE_Failure;
+            }
+            const auto eLastDT = m_aoSteps.back().eOutDT;
+            const int nLastDTSize = GDALGetDataTypeSizeBytes(eLastDT);
+            if (bIsBIPLike)
+            {
+                for (int iY = 0; iY < nBufYSize; ++iY)
+                {
+                    GDALCopyWords64(
+                        m_abyInput.data() + static_cast<size_t>(iY) * nBands *
+                                                nBufXSize * nLastDTSize,
+                        eLastDT, nLastDTSize, pabyData + iY * nLineSpace,
+                        eBufType, GDALGetDataTypeSizeBytes(eBufType),
+                        static_cast<size_t>(nBufXSize) * nBands);
+                }
+            }
+            else
+            {
+                CPLAssert(bIsBSQLike);
+                for (int iBand = 0; iBand < nBands; ++iBand)
+                {
+                    for (int iY = 0; iY < nBufYSize; ++iY)
+                    {
+                        GDALCopyWords64(
+                            m_abyInput.data() +
+                                (static_cast<size_t>(iY) * nBands * nBufXSize +
+                                 iBand) *
+                                    nLastDTSize,
+                            eLastDT, nLastDTSize * nBands,
+                            pabyData + iBand * nBandSpace + iY * nLineSpace,
+                            eBufType, nBufTypeSize, nBufXSize);
+                    }
+                }
+            }
+            return CE_None;
+        }
+    }
+
+    return VRTDataset::IRasterIO(eRWFlag, nXOff, nYOff, nXSize, nYSize, pData,
+                                 nBufXSize, nBufYSize, eBufType, nBandCount,
+                                 panBandMap, nPixelSpace, nLineSpace,
+                                 nBandSpace, psExtraArg);
+}
+
 /*! @endcond */
 
 /************************************************************************/
