@@ -8,23 +8,7 @@
  * Copyright (c) 1999, Frank Warmerdam
  * Copyright (c) 2008-2014, Even Rouault <even dot rouault at spatialys.com>
  *
- * Permission is hereby granted, free of charge, to any person obtaining a
- * copy of this software and associated documentation files (the "Software"),
- * to deal in the Software without restriction, including without limitation
- * the rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons to whom the
- * Software is furnished to do so, subject to the following conditions:
- *
- * The above copyright notice and this permission notice shall be included
- * in all copies or substantial portions of the Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
- * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
- * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
- * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
- * DEALINGS IN THE SOFTWARE.
+ * SPDX-License-Identifier: MIT
  ****************************************************************************/
 
 #include "cpl_port.h"
@@ -44,14 +28,41 @@
 #include "ogr_p.h"
 
 /************************************************************************/
-/*                             OGRPolygon()                             */
+/*        OGRPolygon(double x1, double y1, double x2, double y2)        */
 /************************************************************************/
 
 /**
- * \brief Create an empty polygon.
+ * \brief Construct a rectangular polygon from opposite corner coordinates.
+ *
+ * @since GDAL 3.11
  */
 
-OGRPolygon::OGRPolygon() = default;
+OGRPolygon::OGRPolygon(double x1, double y1, double x2, double y2)
+{
+    auto poLR = std::make_unique<OGRLinearRing>();
+    poLR->addPoint(x1, y1);
+    poLR->addPoint(x1, y2);
+    poLR->addPoint(x2, y2);
+    poLR->addPoint(x2, y1);
+    poLR->addPoint(x1, y1);
+    addRingDirectly(poLR.release());
+}
+
+/************************************************************************/
+/*                OGRPolygon(const OGREnvelope &envelope)               */
+/************************************************************************/
+
+/**
+ * \brief Construct a rectangular polygon from opposite corner coordinates
+ * of a OGREnvelope.
+ *
+ * @since GDAL 3.11
+ */
+
+OGRPolygon::OGRPolygon(const OGREnvelope &envelope)
+    : OGRPolygon(envelope.MinX, envelope.MinY, envelope.MaxX, envelope.MaxY)
+{
+}
 
 /************************************************************************/
 /*                     OGRPolygon( const OGRPolygon& )                  */
@@ -68,12 +79,6 @@ OGRPolygon::OGRPolygon() = default;
 
 OGRPolygon::OGRPolygon(const OGRPolygon &) = default;
 
-/************************************************************************/
-/*                            ~OGRPolygon()                             */
-/************************************************************************/
-
-OGRPolygon::~OGRPolygon() = default;
-
 /************************************************************************/
 /*                     operator=( const OGRPolygon&)                    */
 /************************************************************************/
@@ -103,7 +108,16 @@ OGRPolygon &OGRPolygon::operator=(const OGRPolygon &other)
 OGRPolygon *OGRPolygon::clone() const
 
 {
-    return new (std::nothrow) OGRPolygon(*this);
+    auto ret = new (std::nothrow) OGRPolygon(*this);
+    if (ret)
+    {
+        if (ret->WkbSize() != WkbSize())
+        {
+            delete ret;
+            ret = nullptr;
+        }
+    }
+    return ret;
 }
 
 /************************************************************************/
@@ -279,18 +293,26 @@ OGRLinearRing *OGRPolygon::stealInteriorRing(int iRing)
 }
 
 /*! @cond Doxygen_Suppress */
+
+/************************************************************************/
+/*                            isRingCorrectType()                               */
+/************************************************************************/
+bool OGRPolygon::isRingCorrectType(const OGRCurve *poRing) const
+{
+    return poRing != nullptr && EQUAL(poRing->getGeometryName(), "LINEARRING");
+}
+
 /************************************************************************/
 /*                            checkRing()                               */
 /************************************************************************/
 
-int OGRPolygon::checkRing(OGRCurve *poNewRing) const
+bool OGRPolygon::checkRing(const OGRCurve *poNewRing) const
 {
-    if (poNewRing == nullptr ||
-        !(EQUAL(poNewRing->getGeometryName(), "LINEARRING")))
+    if (!isRingCorrectType(poNewRing))
     {
         CPLError(CE_Failure, CPLE_AppDefined,
                  "Wrong curve type. Expected LINEARRING.");
-        return FALSE;
+        return false;
     }
 
     if (!poNewRing->IsEmpty() && !poNewRing->get_IsClosed())
@@ -302,7 +324,7 @@ int OGRPolygon::checkRing(OGRCurve *poNewRing) const
         if (pszEnvVar != nullptr && !CPLTestBool(pszEnvVar))
         {
             CPLError(CE_Failure, CPLE_AppDefined, "Non closed ring detected.");
-            return FALSE;
+            return false;
         }
         else
         {
@@ -315,7 +337,7 @@ int OGRPolygon::checkRing(OGRCurve *poNewRing) const
         }
     }
 
-    return TRUE;
+    return true;
 }
 
 /*! @endcond */
