@@ -88,6 +88,8 @@ int OGRDXFDataSource::Open(const char *pszFilename, bool bHeaderOnly,
                            CSLConstList papszOptionsIn)
 
 {
+    SetDescription(pszFilename);
+
     osEncoding = CPL_ENC_ISO8859_1;
 
     bInlineBlocks = CPLTestBool(
@@ -821,6 +823,8 @@ bool OGRDXFDataSource::ReadHeaderSection()
         }
 
         oHeaderVariables[l_osName] = szLineBuf;
+        GDALDataset::SetMetadataItem(l_osName.c_str(), szLineBuf,
+                                     "DXF_HEADER_VARIABLES");
     }
     if (nCode < 0)
     {
@@ -857,6 +861,8 @@ bool OGRDXFDataSource::ReadHeaderSection()
             }
 
             oHeaderVariables[l_osName] = szLineBuf;
+            GDALDataset::SetMetadataItem(l_osName.c_str(), szLineBuf,
+                                         "DXF_HEADER_VARIABLES");
         }
         if (nCode < 0)
         {
